/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ExportRulePackagesServlet.java 4905 2009-08-13 13:35:13Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.common.servlet;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.modeler.util.DomUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * A servlets to export rule packages as DRL files (stored in a ZIP archive
 * if more than one). Does not handle GET requests.
 *
 * @author Julien Nicoulaud
 */
public class ExportRulePackagesServlet extends HttpServlet {

    /**
     * Class version ID
     */
    private static final long serialVersionUID = -4729955174780787192L;

    /**
     * Logger
     */
    //private static Log logger = LogFactory.getLog(ExportRulePackagesServlet.class);

    /**
     * Temporary directory where files are stored
     */
    private final String tempDir = System.getProperty("java.io.tmpdir") + File.separatorChar + "EoS.temp" + File.separatorChar;

    /**
     * Files read/write buffer size
     */
    private static final int DEFAULT_BUFFER_SIZE = 2048;

    /**
     * The path where to find the empty rule bundle
     */
    private final String BUNDLE_PROTOTYPE_PATH = "WEB-INF/lib/rules-bundle-prototype.jar";

    /**
     * The path in the generated bundle where the DRL files are stored
     */
    private final String BUNDLE_RULES_PATH = "rules/";


    /**
     * Handle POST requests
     */
    @Override
    protected void doPost(final HttpServletRequest req, final HttpServletResponse res) throws ServletException, IOException {
        try {

            // Set a response content type
            res.setContentType("text/xml");

            // Setup the output stream for the return XML data
            ServletOutputStream out = res.getOutputStream();
            out.println("<response>");

            // Initialize the temporary directory
            File tmpDirFile = new File(tempDir);
            tmpDirFile.mkdirs();

            // Format the date inserted in file names
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss");
            String date = dateFormat.format(new Date());

            // Retrieve the rule packages and create the corresponding files
            List<File> files = new ArrayList<File>();

            //// Convert the request to an XML document
            Node packages = ConvertStringToNode(req.getParameter("packages"));

            //// Step over the root node
            packages = packages.getFirstChild();

            //// For each package
            for (int i = 0; i<packages.getChildNodes().getLength(); i++) {

                // Write it to a file
                Node drlFile = packages.getChildNodes().item(i);
                File file = new File(tempDir + DomUtil.getChild(drlFile,"name").getTextContent() + "_" + date + ".drl");
                FileOutputStream fileOutStream = new FileOutputStream(file);
                fileOutStream.write(DomUtil.getChild(drlFile,"source").getTextContent().getBytes());
                fileOutStream.close();

                // Store it in our list
                files.add(file);
            }

            // Switch case depending on the target output
            String targetOutput = req.getParameter("output");

            // If the user wants to get a bundle
            if (targetOutput.equals("bundle")) {

                // Create a new JAR file
                File jarFile = new File(tempDir + "rules-bundle_" + date + ".jar");
                BufferedInputStream origin = null;
                FileOutputStream dest = new FileOutputStream(jarFile);
                ZipOutputStream jarOut = new ZipOutputStream(new BufferedOutputStream(dest));
                byte data[] = new byte[DEFAULT_BUFFER_SIZE];

                // Retrieve the prototype bundle
                URL protoBundlePath = getClass().getClassLoader().getResource(BUNDLE_PROTOTYPE_PATH);
                File protoBundle = new File(protoBundlePath.getPath());

                // Copy the prototype bundle contents to the new JAR
                ZipInputStream protoBundleIn = new ZipInputStream(new FileInputStream(protoBundle));
                ZipEntry protoBundleEntry = protoBundleIn.getNextEntry();
                while (protoBundleEntry != null) {
                    jarOut.putNextEntry(new ZipEntry(protoBundleEntry));
                    int count;

                    // Special treatment for the manifest file...
                    // FIXME Dirty hack to make sure the bundle symbolic name is unique
                    // (see http://jira.jasmine.objectweb.org/browse/MONITORING-72)
                    if (protoBundleEntry.getName().equals("META-INF/MANIFEST.MF")) {
                        ByteArrayOutputStream manifestFile = new ByteArrayOutputStream();
                        while ((count = protoBundleIn.read(data, 0, DEFAULT_BUFFER_SIZE)) > 0) {
                            manifestFile.write(data, 0, count);
                        }
                        String manifestFileContent = manifestFile.toString();
                        manifestFileContent = manifestFileContent.replaceFirst("Bundle-SymbolicName: ", "Bundle-SymbolicName: " + System.currentTimeMillis());
                        jarOut.write(manifestFileContent.getBytes());
                    }
                    // /FIXME

                    else {
                        while ((count = protoBundleIn.read(data, 0, DEFAULT_BUFFER_SIZE)) > 0) {
                            jarOut.write(data, 0, count);
                        }
                    }
                    protoBundleEntry = protoBundleIn.getNextEntry();
                }
                protoBundleIn.close();

                // Add each rule package file, in the JAR folder defined by BUNDLE_RULES_PATH
                for (int i=0; i<files.size(); i++) {
                   FileInputStream fi = new FileInputStream(files.get(i));
                   origin = new BufferedInputStream(fi, DEFAULT_BUFFER_SIZE);
                   ZipEntry newEntry = new ZipEntry(BUNDLE_RULES_PATH + files.get(i).getName());
                   jarOut.putNextEntry(newEntry);
                   int count;
                   while((count = origin.read(data, 0, DEFAULT_BUFFER_SIZE)) > 0) {
                       jarOut.write(data, 0, count);
                   }
                   origin.close();
                }
                jarOut.close();

                // Send back the reference of the archive
                out.println("<file " +
                                 "path=\"" + jarFile.getAbsolutePath() + "\" " +
                                 "name=\"" + jarFile.getName() + "\" " +
                                 "size=\"" + jarFile.length()/1024 + "\" " +
                            "/>");

            }

            // Else export the source files
            else {
                // If several rule packages uploaded, create an archive
                if (files.size() > 1) {

                    // Create a ZIP file
                    File zipFile = new File(tempDir + "DRL_packages_" + date + ".zip");
                    BufferedInputStream origin = null;
                    FileOutputStream dest = new FileOutputStream(zipFile);
                    ZipOutputStream zipOut = new ZipOutputStream(new BufferedOutputStream(dest));
                    byte data[] = new byte[DEFAULT_BUFFER_SIZE];

                    // Add each rule package file, at the ZIP file root folder
                    for (int i=0; i<files.size(); i++) {
                       FileInputStream fi = new FileInputStream(files.get(i));
                       origin = new BufferedInputStream(fi, DEFAULT_BUFFER_SIZE);
                       ZipEntry entry = new ZipEntry(files.get(i).getName());
                       zipOut.putNextEntry(entry);
                       int count;
                       while((count = origin.read(data, 0, DEFAULT_BUFFER_SIZE)) != -1) {
                          zipOut.write(data, 0, count);
                       }
                       origin.close();
                    }
                    zipOut.close();

                    // Send back the reference of the archive
                    out.println("<file " +
                                     "path=\"" + zipFile.getAbsolutePath() + "\" " +
                                     "name=\"" + zipFile.getName() + "\" " +
                                     "size=\"" + zipFile.length()/1024 + "\" " +
                                "/>");
                }

                // If only one rule package uploaded, send directly the link to
                // the corresponding file
                else {
                    out.println("<file " +
                                    "path=\"" + files.get(0).getAbsolutePath() + "\" " +
                                    "name=\"" + files.get(0).getName() + "\" " +
                                    "size=\"" + files.get(0).length()/1024 + "\" " +
                                "/>");
                }
            }

            // Close off the response XML data and stream
            out.println("</response>");
            out.close();

        } catch (Exception e) {
            e.printStackTrace();
            throw new ServletException(e);
        }
    }

    /**
     * Converts a String to a XML document
     *
     * @param Str the input XML document as String
     * @return an XML root node
     */
    public static Node ConvertStringToNode(final String Str) throws Exception
    {
        Node result=null;
        Document document;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder constructeur = factory.newDocumentBuilder();
        StringBuffer Buffer = new StringBuffer(Str);
        ByteArrayInputStream Bis1 = new ByteArrayInputStream(Buffer.toString().getBytes());
        document = constructeur.parse(Bis1);
        Element root = document.getDocumentElement();
        result = root.getParentNode();
        return result;
    }

}
